@@ -4,10 +4,12 @@ from . import vfs
 from . import core
 from . import params
 from . import config_loader
+from . import vfs_win
 
 from .cluster_processing import *
 from .runner import *
 from .utils import *
 from .surface_processor import *
 from .training import *
-from .predictors import *
\ No newline at end of file
+from .predictors import *
+from .views import *
