@@ -56,12 +56,13 @@ def save_params(params, target_path: Path = None):
 
 
 # ---------- Función común de render (3D+2D) ----------
-def render_dump_to(plotter: QtInteractor, fig: plt.Figure, dump_path: str):
-    """Dibuja celda + puntos igual que load_dump, coloreando por 'Cluster' si existe."""
+# ---------- Función común de render (3D opcional + 2D si fig no es None) ----------
+def render_dump_to(plotter: QtInteractor, fig, dump_path: str):
+    """Dibuja celda + puntos; si fig es None, NO dibuja la vista 2D."""
     pipeline = import_file(dump_path)
     data = pipeline.compute()
 
-    # === Celda desde OVITO: columnas a1,a2,a3 y última columna origen ===
+    # === Celda desde OVITO ===
     M = np.asarray(data.cell.matrix, dtype=float)   # (3x4)
     a1, a2, a3, origin = M[:, 0], M[:, 1], M[:, 2], M[:, 3]
 
@@ -81,7 +82,7 @@ def render_dump_to(plotter: QtInteractor, fig: plt.Figure, dump_path: str):
     pos_prop = data.particles.positions
     positions = pos_prop.array if hasattr(pos_prop, "array") else np.asarray(pos_prop, dtype=float)
 
-    # --- Detectar columna de clúster (varios alias posibles) ---
+    # --- Detectar columna de clúster ---
     cluster_vals = None
     for name in ("Cluster", "cluster", "c_Cluster", "c_cluster", "ClusterID", "cluster_id"):
         if name in data.particles:
@@ -90,13 +91,11 @@ def render_dump_to(plotter: QtInteractor, fig: plt.Figure, dump_path: str):
             cluster_vals = np.asarray(arr).astype(int).reshape(-1)
             break
 
-    # --- Remapeo a 0..K-1 para paleta discreta ---
     cluster_idx = None
     unique_clusters = None
     if cluster_vals is not None and cluster_vals.shape[0] == positions.shape[0]:
         unique_clusters = np.unique(cluster_vals)
         map_idx = {val: i for i, val in enumerate(unique_clusters)}
-        # vectorizado seguro
         cluster_idx = np.vectorize(map_idx.get, otypes=[int])(cluster_vals)
 
     # === Vista 3D ===
@@ -113,7 +112,7 @@ def render_dump_to(plotter: QtInteractor, fig: plt.Figure, dump_path: str):
             render_points_as_spheres=True,
             point_size=8,
             cmap="tab20",
-            show_scalar_bar=False,   # oculto barra para muchos clústeres
+            show_scalar_bar=False,
         )
     else:
         plotter.add_mesh(
@@ -126,28 +125,29 @@ def render_dump_to(plotter: QtInteractor, fig: plt.Figure, dump_path: str):
     plotter.reset_camera()
     plotter.set_scale(1, 1, 1)
 
-    # === Vista 2D ===
-    fig.clf()
-    ax = fig.add_subplot(111)
-    for i, j in edges:
-        x0, y0 = corners[i][0], corners[i][1]
-        x1, y1 = corners[j][0], corners[j][1]
-        ax.plot([x0, x1], [y0, y1], '-', linewidth=1)
+    # === Vista 2D (solo si hay fig) ===
+    if fig is not None:
+        fig.clf()
+        ax = fig.add_subplot(111)
+        for i, j in edges:
+            x0, y0 = corners[i][0], corners[i][1]
+            x1, y1 = corners[j][0], corners[j][1]
+            ax.plot([x0, x1], [y0, y1], '-', linewidth=1)
 
-    if cluster_idx is not None:
-        # Paleta consistente con PyVista
-        ax.scatter(
-            positions[:, 0], positions[:, 1], s=10,
-            c=cluster_idx, cmap="tab20",
-            vmin=0, vmax=len(unique_clusters)-1
-        )
-    else:
-        ax.scatter(positions[:, 0], positions[:, 1], s=10, color="k")
+        if cluster_idx is not None:
+            ax.scatter(
+                positions[:, 0], positions[:, 1], s=10,
+                c=cluster_idx, cmap="tab20",
+                vmin=0, vmax=len(unique_clusters)-1
+            )
+        else:
+            ax.scatter(positions[:, 0], positions[:, 1], s=10, color="k")
+
+        ax.set_xlabel('X'); ax.set_ylabel('Y')
+        ax.set_aspect('equal', 'box')
+        ax.grid(True, linewidth=0.3)
+        fig.canvas.draw()
 
-    ax.set_xlabel('X'); ax.set_ylabel('Y')
-    ax.set_aspect('equal', 'box')
-    ax.grid(True, linewidth=0.3)
-    fig.canvas.draw()
 
 
 # ---------- Widgets de viewers internos ----------
@@ -204,11 +204,13 @@ class DumpViewerWidget(QWidget):
 
 
 class KeyAreaSeqWidget(QWidget):
-    """Viewer para outputs/dump/key_area_{i}.dump con controles de índice."""
+    """Viewer para outputs/dump/key_area_{i}.dump + panel de distribución (results.csv)."""
     def __init__(self, parent=None, pattern: str = "outputs/dump/key_area_{i}.dump"):
         super().__init__(parent)
         self.pattern = pattern
+        self._results_df = None  # cache df results
 
+        # --- Barra superior de navegación por key_area_i.dump ---
         top = QWidget()
         top_l = QHBoxLayout(top)
         self.idx = QSpinBox()
@@ -227,24 +229,56 @@ class KeyAreaSeqWidget(QWidget):
         top_l.addWidget(QLabel("Archivo:"))
         top_l.addWidget(self.path_lbl, 1)
 
-        center = QWidget()
-        center_l = QVBoxLayout(center)
-        self.plotter = QtInteractor(center)
-        center_l.addWidget(self.plotter)
+        # --- Centro: viewer dump (3D + 2D) ---
+        viewer = QWidget()
+        viewer_l = QVBoxLayout(viewer)
+        self.plotter = QtInteractor(viewer)
+        viewer_l.addWidget(self.plotter)
         self.fig = plt.figure(figsize=(4, 4))
         self.canvas = FigureCanvas(self.fig)
-        center_l.addWidget(self.canvas)
+        viewer_l.addWidget(self.canvas)
 
-        root_l = QVBoxLayout(self)
-        root_l.addWidget(top)
-        root_l.addWidget(center, 1)
+        # --- Panel de distribución de poros (results.csv) ---
+        dist_panel = QWidget()
+        dist_l = QVBoxLayout(dist_panel)
 
+        # Acciones / ruta
+        row = QWidget()
+        row_l = QHBoxLayout(row)
+        self.results_btn = QPushButton("Cargar results.csv")
+        self.results_path = QLineEdit(str(Path.cwd() / "outputs" / "csv" / "results.csv"))
+        self.results_path.setReadOnly(True)
+        row_l.addWidget(self.results_btn)
+        row_l.addWidget(self.results_path, 1)
+        dist_l.addWidget(row)
+
+        # Tabla de vista (primeras filas o todo si querés)
+        self.table_results = QTableWidget()
+        self.table_results.setMinimumHeight(160)
+        dist_l.addWidget(QLabel("Vista de resultados (archivo, surface_area, filled_volume, predicted_vacancy):"))
+        dist_l.addWidget(self.table_results)
+
+        # Canvas para gráficos (2x2)
+        self.fig_dist = plt.figure(figsize=(6, 5))
+        self.canvas_dist = FigureCanvas(self.fig_dist)
+        dist_l.addWidget(self.canvas_dist)
+
+        # Conectar señales
         self.btn_prev.clicked.connect(lambda: self._step(-1))
         self.btn_next.clicked.connect(lambda: self._step(+1))
         self.btn_load.clicked.connect(self._load_idx)
+        self.results_btn.clicked.connect(self.load_results_csv)
 
+        # --- Layout raíz de la pestaña ---
+        root_l = QVBoxLayout(self)
+        root_l.addWidget(top)
+        root_l.addWidget(viewer, 1)
+        root_l.addWidget(dist_panel)
+
+        # Intentar autodescubrir primer key_area existente
         self._auto_seed_index()
 
+    # ----------- helpers key_area viewer -----------
     def _pattern_path(self, i: int) -> str:
         return self.pattern.format(i=i)
 
@@ -272,6 +306,103 @@ class KeyAreaSeqWidget(QWidget):
         except Exception as e:
             QMessageBox.critical(self, "Error", str(e))
 
+    # ----------- panel results.csv -----------
+    def _normalize_results_df(self, df: pd.DataFrame) -> pd.DataFrame:
+        # normaliza nombres
+        normmap = {c.lower().strip(): c for c in df.columns}
+        def getcol(*cands):
+            for c in cands:
+                if c in normmap:
+                    return normmap[c]
+            return None
+
+        col_archivo = getcol("archivo", "file", "path")
+        col_area    = getcol("surface_area", "area", "surfacearea")
+        col_vol     = getcol("filled_volume", "volume", "filledvolume")
+        col_vac     = getcol("predicted_vacancy", "vacancy", "vacancys", "vacancys_est", "predicted")
+
+        missing = []
+        if col_archivo is None: missing.append("archivo")
+        if col_area    is None: missing.append("surface_area")
+        if col_vol     is None: missing.append("filled_volume")
+        if col_vac     is None: missing.append("predicted_vacancy")
+        if missing:
+            raise ValueError("Faltan columnas requeridas: " + ", ".join(missing) +
+                             f"\nColumnas disponibles: {list(df.columns)}")
+
+        out = df.rename(columns={
+            col_archivo: "archivo",
+            col_area: "surface_area",
+            col_vol: "filled_volume",
+            col_vac: "predicted_vacancy"
+        }).copy()
+
+        for c in ("surface_area", "filled_volume", "predicted_vacancy"):
+            out[c] = pd.to_numeric(out[c], errors="coerce")
+
+        out = out.dropna(subset=["surface_area", "filled_volume", "predicted_vacancy"])
+        return out
+
+    def load_results_csv(self):
+        try:
+            path = Path(self.results_path.text().strip())
+            if not path.exists():
+                # fallback por si el usuario lo movió
+                path = Path.cwd() / "outputs" / "csv" / "results.csv"
+                self.results_path.setText(str(path))
+            if not path.exists():
+                QMessageBox.warning(self, "No existe", f"No se encontró:\n{path.as_posix()}")
+                return
+
+            df = pd.read_csv(path)
+            df = self._normalize_results_df(df)
+            self._results_df = df
+
+            # ---- Poblar tabla con columnas clave ----
+            cols = ["archivo", "surface_area", "filled_volume", "predicted_vacancy"]
+            self.table_results.clear()
+            self.table_results.setColumnCount(len(cols))
+            self.table_results.setRowCount(len(df))
+            self.table_results.setHorizontalHeaderLabels(cols)
+            for i in range(len(df)):
+                for j, col in enumerate(cols):
+                    self.table_results.setItem(i, j, QTableWidgetItem(str(df.iat[i, df.columns.get_loc(col)])))
+            self.table_results.resizeColumnsToContents()
+
+            # ---- Dibujar gráficos (2x2) ----
+            self.fig_dist.clf()
+            axs = self.fig_dist.subplots(2, 2)
+
+            # Hist surface_area
+            axs[0, 0].hist(df["surface_area"].values, bins="auto")
+            axs[0, 0].set_title("Distribución: surface_area")
+            axs[0, 0].set_xlabel("surface_area"); axs[0, 0].set_ylabel("Frecuencia")
+
+            # Hist filled_volume
+            axs[0, 1].hist(df["filled_volume"].values, bins="auto")
+            axs[0, 1].set_title("Distribución: filled_volume")
+            axs[0, 1].set_xlabel("filled_volume"); axs[0, 1].set_ylabel("Frecuencia")
+
+            # Conteo predicted_vacancy (redondeado a entero)
+            vac_counts = df["predicted_vacancy"].round(0).astype(int).value_counts().sort_index()
+            axs[1, 0].bar(vac_counts.index.astype(str), vac_counts.values)
+            axs[1, 0].set_title("Conteo: predicted_vacancy")
+            axs[1, 0].set_xlabel("predicted_vacancy"); axs[1, 0].set_ylabel("Cantidad de poros")
+
+            # Scatter SA vs FV (tamaño ~ vacancias)
+            s = df["predicted_vacancy"].values
+            scale = 15 / (np.nanpercentile(s, 90) + 1e-9)
+            size = 10 + scale * s
+            axs[1, 1].scatter(df["surface_area"].values, df["filled_volume"].values, s=size)
+            axs[1, 1].set_title("surface_area vs filled_volume (size ~ vacancias)")
+            axs[1, 1].set_xlabel("surface_area"); axs[1, 1].set_ylabel("filled_volume")
+
+            self.fig_dist.tight_layout()
+            self.canvas_dist.draw()
+
+        except Exception as e:
+            QMessageBox.critical(self, "Error al cargar results.csv", str(e))
+
 
 # ---------- Ventana principal ----------
 class SettingsWindow(QMainWindow):
@@ -366,16 +497,15 @@ class SettingsWindow(QMainWindow):
         # Botones
         btn_save = QPushButton("Save Settings"); btn_save.clicked.connect(self.save_settings_and_notify)
         btn_run = QPushButton("Run VacancyAnalysis"); btn_run.clicked.connect(self.run_vacancy_analysis)
-        btn_total = QPushButton("Total Vacancies"); btn_total.clicked.connect(self.show_total_vacancies)  # ⬅️ nuevo
+        btn_total = QPushButton("Total Vacancies"); btn_total.clicked.connect(self.show_total_vacancies)
 
         hb = QHBoxLayout(); hb.addWidget(btn_save); hb.addWidget(btn_run)
-
         form_layout.addRow(hb)
 
         hb = QHBoxLayout()
         hb.addWidget(btn_save)
         hb.addWidget(btn_run)
-        hb.addWidget(btn_total)  # ⬅️ nuevo
+        hb.addWidget(btn_total)
         form_layout.addRow(hb)
 
         # Panel izquierdo (controles)
@@ -386,7 +516,7 @@ class SettingsWindow(QMainWindow):
         # Panel derecho con pestañas (Main / Viewer 1 / Key Areas)
         tabs = QTabWidget()
 
-        # --- Tab Main (tu viewer original) ---
+        # --- Tab Main ---
         main_tab = QWidget(); main_layout = QVBoxLayout(main_tab)
         self.plotter = QtInteractor(main_tab); main_layout.addWidget(self.plotter)
         self.fig = plt.figure(figsize=(4, 4)); self.canvas = FigureCanvas(self.fig); main_layout.addWidget(self.canvas)
@@ -397,7 +527,7 @@ class SettingsWindow(QMainWindow):
         self.viewer1 = DumpViewerWidget()
         tabs.addTab(self.viewer1, "Viewer 1")
 
-        # --- Tab Key Areas ---
+        # --- Tab Key Areas (ahora con distribución) ---
         self.viewer2 = KeyAreaSeqWidget()
         tabs.addTab(self.viewer2, "Key Areas")
 
@@ -451,6 +581,7 @@ class SettingsWindow(QMainWindow):
             widget = getattr(self, f"spin_{key}".replace(' ', '_'))
             cfg[key] = widget.value()
         return save_params(self.params)
+
     def show_total_vacancies(self):
         """Abre una ventana con el total de vacancias de outputs/csv/results.csv."""
         try:
@@ -461,8 +592,6 @@ class SettingsWindow(QMainWindow):
                 return
 
             df = pd.read_csv(path)
-
-            # columnas candidatas donde puede estar el número de vacancias por fila
             candidates = ["predicted_vacancy", "vacancys_est", "vacancys", "predicted", "vacancy"]
             col = next((c for c in candidates if c in df.columns), None)
             if col is None:
@@ -475,18 +604,13 @@ class SettingsWindow(QMainWindow):
 
             vals = pd.to_numeric(df[col], errors='coerce').fillna(0.0)
             total = float(vals.sum())
-            total_int = int(np.ceil(total))  # redondeo hacia arriba por si hay decimales
+            total_int = int(np.ceil(total))  # redondeo hacia arriba
 
-            # Mensaje bonito
             msg = QMessageBox(self)
             msg.setWindowTitle("Total de vacancias")
-            msg.setText(
-                f"<h2 style='margin:0'>Total de vacancias: {total_int}</h2>"
-                f"<p style='margin-top:8px'>Archivo: {path.as_posix()}<br>"
-                f"Columna usada: <b>{col}</b><br>"
-                f"Suma exacta: {total:.2f}</p>"
-            )
+            msg.setText(f"<h2 style='margin:0'>Total de vacancias: {total_int}</h2>")
             msg.exec()
+            print(f"Vacancias totales: {total_int}")
 
         except Exception as e:
             QMessageBox.critical(self, "Error calculando total", str(e))
@@ -550,7 +674,30 @@ class SettingsWindow(QMainWindow):
                 vfs.VacancyAnalysis()
             self.progress.setRange(0, 100)
             self.progress.setValue(100)
-            self.log_output.setPlainText(buf.getvalue())
+
+            # Log estándar
+            log_text = buf.getvalue()
+
+            # Calcular total vacancias y añadirlo al log
+            try:
+                path = Path.cwd() / 'outputs' / 'csv' / 'results.csv'
+                if path.exists():
+                    df = pd.read_csv(path)
+                    candidates = ["predicted_vacancy", "vacancys_est", "vacancys", "predicted", "vacancy"]
+                    col = next((c for c in candidates if c in df.columns), None)
+                    if col:
+                        vals = pd.to_numeric(df[col], errors='coerce').fillna(0.0)
+                        total_int = int(np.ceil(vals.sum()))
+                        log_text += f"\n\n VACANCIES: {total_int}\n"
+                    else:
+                        log_text += "\n[INFO] No se encontró columna de vacancias en results.csv\n"
+                else:
+                    log_text += "\n[INFO] No existe outputs/csv/results.csv\n"
+            except Exception as e:
+                log_text += f"\n[ERROR] Fallo al calcular total de vacancias: {e}\n"
+
+            self.log_output.setPlainText(log_text)
+
             QMessageBox.information(self, "Análisis completado", "VacancyAnalysis terminó correctamente.")
             self._refresh_csv_list()
             dump_path = Path.cwd() / 'outputs' / 'dump' / 'key_areas.dump'
